#ifndef SAMURAIAADetectorConstruction_H
#define SAMURAIAADetectorConstruction_H 1

class G4LogicalVolume;
class G4VPhysicalVolume;
class SAMURAIAADetectorConstructionMessenger;
class DipoleConstruction;
class TargetConstruction;
class MINOSTargetChamberConstruction;
class FDC1Construction;
class FDC2Construction;
class HODF24Construction;
class ExitWindowC2Construction;
class ExitWindowNConstruction;
class NEBULAConstruction;
class NeuLANDConstruction;
class VacuumUpstreamConstruction;
class VacuumDownstreamConstruction;
class WallConstruction;

class G4VSensitiveDetector;

#include "G4VUserDetectorConstruction.hh"
#include "G4ThreeVector.hh"

class SAMURAIAADetectorConstruction : public G4VUserDetectorConstruction
{
public:

  SAMURAIAADetectorConstruction();
  ~SAMURAIAADetectorConstruction();

  void UpdateGeometry();

  G4VPhysicalVolume* Construct();

  void SetFillAir(bool tf){fFillAir = tf;}
               
private:
  bool fFillAir;
    
  // Physical volumes
  G4VPhysicalVolume* fTarget_phys;

  SAMURAIAADetectorConstructionMessenger *fDetectorConstructionMessenger;

  DipoleConstruction *fDipoleConstruction;
  TargetConstruction *fTargetConstruction;
  MINOSTargetChamberConstruction *fMINOSTargetChamberConstruction;
  FDC1Construction   *fFDC1Construction;
  FDC2Construction   *fFDC2Construction;
  HODF24Construction    *fHODConstruction;
  ExitWindowC2Construction *fExitWindowC2Construction;
  ExitWindowNConstruction *fExitWindowNConstruction;
  NEBULAConstruction *fNEBULAConstruction;
  NeuLANDConstruction *fNeuLANDConstruction;
  VacuumUpstreamConstruction *fVacuumUpstreamConstruction;
  VacuumDownstreamConstruction *fVacuumDownstreamConstruction;
  WallConstruction *fWallConstruction;

  G4ThreeVector fTargetPos;

  G4VSensitiveDetector* fFDC1SD;
  G4VSensitiveDetector* fWindowHoleSD;
  G4VSensitiveDetector* fFDC2SD;
  G4VSensitiveDetector* fHODSD;
  G4VSensitiveDetector* fNeutronWinSD;
  G4VSensitiveDetector* fNEBULASD;

};

#endif

